import java.util.*;
import java.io.*;

public class Client {
    public static void main(String[] args) {
        System.out.println("| n | set | ArrayList | LinkedList | BinarySearchTree |");

        int[] setSizes = {3, 100, 1000, 10000, 100000};
        for (int i = 0; i < setSizes.length; i++) {
            long[] times = containsTest(setSizes[i]);
            System.out.println(Arrays.toString(times));
        }

        // Will generate timings for both add and contains based on the given "max" n value and increments "countBy"
        // the current values took Kasey's laptop roughly 10 minutes to run
        csvData(30000,100);
    }

    public static long[] addTest(int n) {
        long[] times = new long[4];
        times[0] = n;
        long start = System.nanoTime();
        int[] dataSet = randomDataSet(n, 10);
        long finish = System.nanoTime();
        //times[1] = finish - start; time to generate the random set of numbers

        ArrayIntList arrList = new ArrayIntList();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            arrList.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[1] = finish - start;

        LinkedIntList linkList = new LinkedIntList();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            linkList.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[2] = finish - start;

        IntSearchTree bst = new IntSearchTree();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            bst.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[3] = finish - start;

        return times;
    }

    public static long[] containsTest(int n) {
        long[] times = new long[3];
        //times[0] = n;
        long start = System.nanoTime();
        int[] dataSet = randomDataSet(n, 100);
        //int valToFind = dataSet[dataSet.length-1]; // random val to find, may be duplicates
        int valToFind = -1; // guarnteed not in collection, worst case runtime
        long finish = System.nanoTime();
        //times[1] = finish - start; time to generate random set of numbers

        ArrayIntList arrList = new ArrayIntList();
        for (int i = 0; i < n; i++) {
            arrList.add(dataSet[i]);
        }
        start = System.nanoTime();
        arrList.contains(valToFind);
        finish = System.nanoTime();
        times[0] = finish - start;

        LinkedIntList linkList = new LinkedIntList();
        for (int i = 0; i < n; i++) {
            linkList.add(dataSet[i]);
        }
        start = System.nanoTime();
        linkList.contains(valToFind);
        finish = System.nanoTime();
        times[1] = finish - start;

        IntSearchTree bst = new IntSearchTree();
        for (int i = 0; i < n; i++) {
            bst.add(dataSet[i]);
        }
        start = System.nanoTime();
        bst.contains(valToFind);
        finish = System.nanoTime();
        times[2] = finish - start;

        return times;
    }
    public static int[] randomDataSet(int n, int range) {
        Random rand = new Random();
        int[] dataSet = new int[n];
        for (int i = 0; i < n; i++) {
            dataSet[i] = rand.nextInt(range);
        }
        return dataSet;
    }

    public static void csvData(int max, int countBy) {
        try {
            File resultsFile = new File("timeTest.csv");
            PrintWriter results = new PrintWriter(resultsFile);
            results.print("n, ArrayIntList add, LinkedIntList add, IntSearchTree add, ArrayIntList contains, LinkedIntList contains, IntSearchTree contains");
            results.println();
            for (int i = 0; i <= max; i+=countBy) {
                long[] addTimes = addTest(i);
                long[] containsTimes = containsTest(i);

                results.print(addTimes[0]);
                for (int j = 1; j < addTimes.length; j++) {
                    results.print(", " + addTimes[j]);
                }
                for (int j = 0; j < containsTimes.length; j++) {
                    results.print(", " + containsTimes[j]);
                }
                results.println();
            }
            results.close();
        } catch (IOException e) {
            System.out.println("file error");
        }
    }
}