import java.util.*;

// This class represents a tree of integers
public class IntTree {
    private IntTreeNode overallRoot;

    // Constructs the tree shown
    public IntTree() {
        overallRoot = null;
    }

    public IntTree(int[] values) {
        overallRoot = constructorHelper(values, 0);
    }
    private IntTreeNode constructorHelper(int[] values, int i) {
        if (i < values.length) {
            IntTreeNode root = new IntTreeNode(values[i]);
            root.left = constructorHelper(values, i*2 + 1);
            root.right = constructorHelper(values, i*2 + 2);
            return root;
        } else {
            return null;
        }
    }

    public int size() {
        return size(overallRoot);
    }

    private int size(IntTreeNode root) {
        if (root == null) {
            return 0;
        } else {
            return 1 + size(root.left) + size(root.right);
        }
    }

    // adds the given value at the next available location in the tree
    // in level order
    public void add(int value) {
        overallRoot = add(value, overallRoot);
    }
    private IntTreeNode add(int value, IntTreeNode root) {
        if (root == null) {
            return new IntTreeNode(value);
        }
        Queue<IntTreeNode> q = new LinkedList<IntTreeNode>();
        q.add(root);
        // level order traversal looking for empty spot
        while (!q.isEmpty()) {
            IntTreeNode temp = q.remove();
            if (temp.left == null) { // fill in first empty left spot
                temp.left = new IntTreeNode(value);
                return temp;
            } else {
                q.add(temp.left);
            }
            if (temp.right == null) { // if no empty left, fill in a right spot
                temp.right = new IntTreeNode(value);
                return temp;
            } else {
                q.add(temp.right);
            }
        }
        return root;
    }

    // prints out all data contained in the tree on a single line
    // based on a preorder traversal sequence
    public void preOrderTraversal() {
        preOrderTraversal(overallRoot);
    }
    private void preOrderTraversal(IntTreeNode root) {
        if (root != null) {
            System.out.print(root.data + " ");
            preOrderTraversal(root.left);
            preOrderTraversal(root.right);
        }
    }

    // prints out all data contained in the tree on a single line
    // based on an inorder traversal sequence
    public void inOrderTraversal() {
        inOrderTraversal(overallRoot);
    }
    private void inOrderTraversal(IntTreeNode root) {
        if (root != null) {
            preOrderTraversal(root.left);
            System.out.print(root.data + " ");
            preOrderTraversal(root.right);
        }
    }

    // prints out all data contained in the tree on a single line
    // based on a postorder traversal sequence
    public void postOrderTraversal() {
        postOrderTraversal(overallRoot);
    }
    private void postOrderTraversal(IntTreeNode root) {
        if (root != null) {
            preOrderTraversal(root.left);
            preOrderTraversal(root.right);
            System.out.print(root.data + " ");
        }
    }

    // prints out all data contained in the tree, comma separated
    // using paranthesis to surround each sub tree 
    // print "empty" for any missing children nodes    
    // ex: (6, (3, 1, 4), (2, 6, 4))
    public String toString() {
        return toString(overallRoot);
    }
    private String toString(IntTreeNode root) {
        if (root == null) {
            return "empty";
        } else if (root.left == null && root.right == null) {
            return "" + root.data;
        } else {
            String result = "(" + root.data + ", ";
            result += toString(root.left);
            result += ", " + toString(root.right) + ")";
            return result;
        }
    }

    // Class that represents a single node in the tree
    private static class IntTreeNode {
        public int data; // data stored at this node
        public IntTreeNode left; // reference to left subtree
        public IntTreeNode right; // reference to right subtree

        // Constructs a leaf node with the given data.
        public IntTreeNode(int data) {
            this(data, null, null);
        }

        // Constructs a leaf or branch node with the given data and links.
        public IntTreeNode(int data, IntTreeNode left, IntTreeNode right) {
            this.data = data;
            this.left = left;
            this.right = right;
        }
    }
}